
# coding: utf-8

# In[ ]:

__version_info__ = (0, 0, 14)
__version__ = '.'.join(map(str, __version_info__))

from .chain import Chain, this, _X
from .magic import magical
from .dispatch import Dispatch
from toolz.curried import *
import toolz.curried

__all__ = [
    'magical', 'Chain', 'this', 'Dispatch', '_X', *pipe(
        toolz.curried, dir, filter(
            complement(lambda s: s.startswith('_'))
        ), list
    )
]





# coding: utf-8

# In[1]:

from collections import OrderedDict
from typing import Iterable, Any

__all__ = ['Dispatch']

class Dispatch(OrderedDict):
    """An object that provides multiple dispatch when it is called.
    """
    def __call__(self, *args, **kwargs):
        for types, fn in self.items():
            if not isinstance(types, Iterable):
                types = tuple([types])
            if len(args) == len(types):
                for arg, type_ in zip(args, types):
                    if type_ != Any and not isinstance(arg, type_): break
                else:
                    return fn(*args)
        raise TypeError("Type(s) not found")

